
/**
 * init buffers to create a square
 * @param {any} gl the web gl context
 * @param {Array<number>} positions the position buffer to be loaded
 * @param {Array<number>} indices the index buffer to be loaded
 * @param {Array<number>} normals the normal buffer to be loaded
 * @return {any} the buffers
 */
export function initBuffers(
	gl: any,
	positions: Array<number>,
	indices: Array<number>,
	normals: Array<number>) {
	const positionBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, positionBuffer);
	gl.bufferData(
		gl.ARRAY_BUFFER,
		new Float32Array(positions),
		gl.STATIC_DRAW);

	const normalBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, normalBuffer);
	gl.bufferData(
		gl.ARRAY_BUFFER,
		new Float32Array(normals),
		gl.STATIC_DRAW);

	const myColors = [
		[1.0, 1.0, 1.0, 1.0],
		[1.0, 0.0, 0.0, 1.0],
		[0.0, 1.0, 0.0, 1.0],
		[0.0, 0.0, 1.0, 1.0],
		[1.0, 1.0, 0.0, 1.0],
		[1.0, 0.0, 1.0, 1.0],
		[0.0, 0.0, 0.0, 1.0],
		[0.0, 1.0, 1.0, 1.0],
	];

	let faceColors: any = [];

	for (let i = 0; i < 70; i++) {
		faceColors = faceColors.concat(myColors);
	}
	// Convert the array of colors into a table for all the vertices.

	const colors: any = [];

	for (let j = 0; j < indices.length; ++j) {
		const c = [
			myColors[Math.floor(Math.random() * 8)],
			faceColors[Math.floor(Math.random() * 8)],
			faceColors[Math.floor(Math.random() * 8)],
			faceColors[Math.floor(Math.random() * 8)],
		];
		// Repeat each color four times for the four vertices of the face
		// colors = colors.concat([1.0, 1.0, 1.0, 1.0]);
		colors.push(c[0][0]);
		colors.push(c[0][1]);
		colors.push(c[0][2]);
		colors.push(c[0][3]);
	}

	const colorBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, colorBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(colors), gl.STATIC_DRAW);

	const indexBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, indexBuffer);
	gl.bufferData(gl.ELEMENT_ARRAY_BUFFER,
		new Uint16Array(indices), gl.STATIC_DRAW);
	// Now send the element array to GL

	return {
		position: positionBuffer,
		color: colorBuffer,
		indices: indexBuffer,
		normals: normalBuffer,
	};
}

/**
 * init buffers to create a square
 * @param {any} gl the web gl context
 * @param {any} buffers the buffers to delete
 */
export function deleteBuffers(gl: any, buffers: any) {
	gl.deleteBuffer(buffers.color);
	gl.deleteBuffer(buffers.position);
	gl.deleteBuffer(buffers.normals);
	gl.deleteBuffer(buffers.indices);
}
